@@ -11,11 +11,12 @@ param = ['abc', 123, 12.5, 0, 'foo', 'foo', '']
 def test_set(red, k):
     assert red.set('sam', k)
     assert red.get('sam') == k
+    red.delete('sam')
     
 
 # @pytest.mark.focus
 def test_list(red):
-    red.delete(red.formatkey('many'))
+    red.delete('many')
     red.set('many', ['a'])
     red.set('many', ['b'])
     red.set('many', ['c'], insert='start')
@@ -28,22 +29,41 @@ def test_list(red):
     red.set('many', ['', 'meh'])
     assert red.llen(red.formatkey('many')) == 8
     assert (red.get('many')) == ['d', 'c', 'a', 'b', 'foo', 'bar', '', 'meh']
+    red.delete('many')
 
 
 # @pytest.mark.focus
 def test_hash(red):
-    red.delete(red.formatkey('user'))
+    red.delete('user')
     red.set('user', dict(age=34, username='enchance', gender='m'))
     assert red.get('user') == dict(age=34, username='enchance', gender='m')
     assert red.get('user', only='username') == dict(username='enchance')
     assert red.get('user', only=['age', 'gender']) == dict(age=34, gender='m')
+    red.delete('user')
+
+
+# @pytest.mark.focus
+def test_get_none(red):
+    red.delete('xxxyyyzzz')
+    assert red.get('xxxyyyzzz') is None
+    assert red.get('xxxyyyzzz', 'foo') == 'foo'
+    assert red.get('xxxyyyzzz', []) == []
+    assert red.get('xxxyyyzzz', [123, 345]) == [123, 345]
+    assert red.get('xxxyyyzzz', [123, 345]) == [123, 345]
+    assert red.get('xxxyyyzzz', {}) == {}
+    assert red.get('xxxyyyzzz', dict(a='b')) == dict(a='b')
+    assert red.get('xxxyyyzzz', 123) == 123
+    assert red.get('xxxyyyzzz', True)
+    assert not red.get('xxxyyyzzz', False)
+    
 
 
 # @pytest.mark.focus
 def test_set_data(red):
-    red.delete(red.formatkey('norepeat'))
+    red.delete('norepeat')
     red.set('norepeat', {'b', 'a', 'c', 'd', 'a'})
     assert red.get('norepeat') == {'d', 'a', 'b', 'c'}   # unordered of course
+    red.delete('norepeat')
 
 
 param = [
@@ -51,27 +71,30 @@ param = [
     ('three', dict(age=34, username='sally', gender='f'), ['one', 'two', 'three'], 3)
 ]
 @pytest.mark.parametrize('key, val, check, out', param)
-@pytest.mark.focus
+# @pytest.mark.focus
 def test_exists(red, key, val, check, out):
     red.set(key, val)
     assert red.exists(*check) == out
     assert isinstance(out, int)
     
+    if check == ['one', 'two', 'three']:
+        red.delete(*check)
+
 
 # @pytest.mark.focus
-def test_overwrite(red, nooverwrite):
-    assert red.set('a', 'a')
-    assert red.set('a', 1)
-    assert red.set('a', 12.5)
-    assert red.set('a', [12, 56])
-    assert red.set('a', dict(foo='bar'))
+def test_overwrite(nooverwrite):
+    assert nooverwrite.set('a', 'a')
     
     with pytest.raises(ResponseError):
         nooverwrite.set('a', [1])
+        assert nooverwrite.set('a', 1)
+        assert nooverwrite.set('a', 12.5)
+        assert nooverwrite.set('a', [12, 56])
+        assert nooverwrite.set('a', dict(foo='bar'))
+        
+    nooverwrite.delete('a')
     
 
-
-
 # @pytest.mark.focus
 def test_exception(red):
     with pytest.raises(ValidationError):
@@ -100,4 +123,32 @@ def test_exception(red):
     except ValidationError as e:
         assert e.message == 'This is it.'
         # ic(e.message)
-    
\ No newline at end of file
+    red.delete('fail')
+
+
+param = [
+    ('ab', 'foo'), ('ab', 1), ('ab', [4, 5]), ('ab', dict(a=1, b=2)),
+    (['hello', 'world'], ['foo', 'bar']), (['hello', 'world', 'x'], ['foo', 'bar', 123]),
+]
+@pytest.mark.parametrize('key, val', param)
+# @pytest.mark.focus
+# @pytest.mark.skip
+def test_delete(red, key, val):
+    if isinstance(key, str):
+        red.set(key, val)
+        v = red.get(key)
+        assert v == val
+        ret = red.delete(key)
+        assert ret == 1
+        v = red.get(key)
+        assert v is None
+        ret = red.delete(key)
+        assert ret == 0
+    else:
+        for idx, k in enumerate(key):
+            red.set(k, val[idx])
+            assert red.get(k) == val[idx]
+        ret = red.delete(*key)
+        assert ret == len(key)
+        for idx, k in enumerate(key):
+            assert red.get(k) is None
